#include <iostream>
#include <vector>
#include <string.h>
#include <cstdio>
#include <map>
#include <queue>
using namespace std;

typedef std::pair <int, int> PosContainer;
typedef std::vector <int> ColorLogContainer;
typedef std::map <int, int> ColorStatisticsContainer;

const unsigned long long MAX_COLORS = 503;
const int MAXN = 351;

int next_x[5] = {0, -1, 0, 0, 1};
int next_y[5] = {0, 0, -1, 1, 0};

int N, M;

class Node {
    public:

        vector < vector <short int> > table;
        int fitness;
        ColorLogContainer color_log;

        Node(){}

        Node(Node& other) {
            table = other.table;
            fitness = other.fitness;
            color_log = other.color_log;
        }

        Node(vector < vector <short int> > other_table, int other_fitness, ColorLogContainer other_color_log):
            table(other_table), fitness(other_fitness), color_log(other_color_log) {
        }

        bool operator < (const Node& other_node) const {
            return fitness < other_node.fitness;
        }

        unsigned long long getHash() const {
            unsigned long long value = 0;
            for (int i = 0; i < table.size(); i++) {
                for (int j = 0; j < table[i].size(); j++) {
                    value *= MAX_COLORS;
                    value += table[i][j];
                }
            }
            return value;
        }

        int getNumberOfColors() {
            int ans = 0;
            bool used[MAX_COLORS];
            memset(used, 0, sizeof(used));
            for (int i = 0; i < this->table.size(); i++) {
                for (int j = 0; j < this->table[i].size(); j++) {
                    if (used[table[i][j]]) {
                        used[table[i][j]] = true;
                        ans++;
                    }
                }
            }
            return ans;
        }

        ~Node() {
            table.clear();
        }
};

class Solution {
    public:
        int k;
        PosContainer pos;
        ColorLogContainer color_log;

        Solution() {
            k = 350 * 350;

        }

        Solution(int other_k, PosContainer other_pos, ColorLogContainer other_color_log): k(other_k), pos(other_pos),
            color_log(other_color_log) {

        }

        ~Solution() {
            color_log.clear();
        }

        bool operator < (const Solution& other_sol) const {
            return k < other_sol.k;
        }

        void print() {
            printf("%d\n", k);
            printf("%d %d\n", pos.first, pos.second);
            for (int i = 0; i < color_log.size(); i++) {
                printf("%d\n", color_log[i]);
            }
        }
};

std::vector <PosContainer> getField(PosContainer pos, Node* node) {
    std::vector <PosContainer> result;
    bool used[MAXN][MAXN];
    memset(used, 0, sizeof(used));
    int current_color = node->table[pos.first][pos.second];
    std::queue <PosContainer> q;
    PosContainer current;
    q.push(pos);
    while(!q.empty()) {
        current = q.front();
        q.pop();
        result.push_back(current);
        used[current.first][current.second] = true;
        for (int i = 1; i < 5; i++) {
            PosContainer next;
            next.first = current.first + next_x[i];
            next.second = current.second + next_y[i];
            if (next.first < 0 || next.first >= N || next.second < 0 || next.second >= M) {
                continue;
            }

            if (node->table[next.first][next.second] != current_color) {
                continue;
            }

            if (used[next.first][next.second]) {
                continue;
            }

            used[next.first][next.second] = true;
            q.push(next);
        }
    }
    return result;
}

ColorStatisticsContainer getContures(PosContainer pos, Node* node) {
    /*for (int i = 0; i < node->table.size(); i++) {
        for (int j = 0; j < node->table[i].size(); j++) {
            cout << node->table[i][j] << " ";

        }
        cout << endl;
    }*/
    ColorStatisticsContainer result;
    bool used[MAXN][MAXN];
    memset(used, 0, sizeof(used));
    int current_color = node->table[pos.first][pos.second];
    std::queue <PosContainer> q;
    PosContainer current;
    q.push(pos);
    while(!q.empty()) {
        current = q.front();
        q.pop();
        //cout << "Node: " << current.first << " " << current.second << std::endl;
        used[current.first][current.second] = true;
        for (int i = 1; i < 5; i++) {
            PosContainer next;
            next.first = current.first + next_x[i];
            next.second = current.second + next_y[i];
            if (next.first < 0 || next.first >= N || next.second < 0 || next.second >= M) {
                continue;
            }

            if (node->table[next.first][next.second] != current_color) {
                if (!used[next.first][next.second]) {
                    std::vector <PosContainer> coleagues = getField(next, node);
                    result[node->table[next.first][next.second]] += coleagues.size();
                    for (int j = 0; j < coleagues.size(); j++) {
                        used[coleagues[j].first][coleagues[j].second] = true;
                    }
                }
                continue;
            }

            if (used[next.first][next.second]) {
                continue;
            }

            used[next.first][next.second] = true;
            q.push(next);
        }
    }
    return result;
}


class Algorithm {
    public:
        virtual Solution* solve(PosContainer pos, int appoximation, Node* initial_state, time_t time_to_work) = 0;
        virtual ~Algorithm() {
        }
};

class Dummy : public Algorithm {
    private:

    public:
        virtual Solution* solve(PosContainer pos, int appoximation, Node* initial_state, clock_t time_to_work) {
            int color_stats[MAX_COLORS];
            memset(color_stats, 0, sizeof(color_stats));
            Node* temp = new Node(initial_state->table, initial_state->fitness, initial_state->color_log);
            Solution* sol = new Solution();
            sol->k = 0;
            sol->pos = pos;
            while (true) {
                if (sol->k >= appoximation) {
                    return NULL;
                }
                ColorStatisticsContainer bound_colors = getContures(pos, temp);
                if (bound_colors.size() == 0) {
                    break;
                }
                std::vector <PosContainer> field = getField(pos, temp);
                ColorStatisticsContainer::iterator lvl;
                int maxi = 0;
                int max_color = 0;
                for (lvl = bound_colors.begin(); lvl != bound_colors.end(); lvl++) {
                    if (maxi < (*lvl).second) {
                        maxi = (*lvl).second;
                        max_color = (*lvl).first;
                    }
//                    cout << (*lvl).first << " " << (*lvl).second << std::endl;
                }

//                cout << "--------------" << std::endl;
                sol->color_log.push_back(max_color);
                for (int i = 0; i < field.size(); i++) {
                    temp->table[field[i].first][field[i].second] = max_color;
                }
                sol->k++;
            }
            delete temp;
            return sol;
        }
        ~Dummy() {

        }
};

class Astar : public Algorithm {
    private:
    public:
        virtual Solution* solve(PosContainer pos, int appoximation, Node* initial_state, clock_t time_to_work) {

            return NULL;
        }
};

class BranchAndBound : public Algorithm {
    private:
        clock_t begin;
        int appoximation;
        Solution* sol;
        PosContainer pos;
        void Dfs(Node* state, int level, clock_t time_to_work) {
            if ((clock() - begin) > time_to_work) return;
            if (level >= this->appoximation) {
                return;
            }
            ColorStatisticsContainer bound_colors = getContures(this->pos, state);
            if (bound_colors.size() == 0) {
                if (sol == NULL) {
                    sol = new Solution();
                }
                appoximation = level;
                sol->color_log = state->color_log;
                return;
            }
            std::vector <PosContainer> field = getField(this->pos, state);
            int current_color = state->table[field[0].first][field[0].second];
            if ((clock() - begin) > time_to_work) return;

            ColorStatisticsContainer::iterator lvl;
            int maxi = 0;
            int max_color = 0;
            for (lvl = bound_colors.begin(); lvl != bound_colors.end(); lvl++) {
                if ((clock() - begin) > time_to_work) return;
                for (int i = 0; i < field.size(); i++) {
                    state->table[field[i].first][field[i].second] = (*lvl).second;
                }

                state->color_log.push_back((*lvl).second);
                Dfs(state, level + 1, time_to_work);

                state->color_log.pop_back();
                for (int i = 0; i < field.size(); i++) {
                    state->table[field[i].first][field[i].second] = current_color;
                }
//                    cout << (*lvl).first << " " << (*lvl).second << std::endl;
            }

        }
    public:
        virtual Solution* solve(PosContainer pos, int appoximation, Node* initial_state, clock_t time_to_work) {
            this->begin = clock();
            this->pos = pos;
            this->appoximation = appoximation;
            sol = NULL;
            Node* temp_state = new Node((*initial_state));
            Dfs(temp_state, 0, time_to_work);
            if (sol != NULL) {
                sol->pos = pos;
                sol->k = sol->color_log.size();
            }

            return sol;
        }
};

class StrategyPattern {
    private:
        Algorithm* algorithm;
        Solution* best_solution;
        Node* initial_state;
    public:
        StrategyPattern(Node* other_initial_state): initial_state(other_initial_state) {
            best_solution = new Solution();
            algorithm = NULL;
        }

        void setAlgorithm(Algorithm* other_algorithm) {
            algorithm = other_algorithm;
        }

        void optimize(clock_t time_to_work) {
            for (int i = 0; i <  initial_state->table.size(); i++) {
                for (int j = 0; j < initial_state->table[i].size(); j++) {
                    Solution* new_sol = algorithm->solve(std::make_pair(i, j), best_solution->k, initial_state, time_to_work);
                    if (new_sol != NULL) {
                        delete best_solution;
                        best_solution = new_sol;
                    }
//                    break;
                }
//                break;
            }
        }

        Solution* getBest() {
            return best_solution;
        }
};

Node* input() {
    scanf("%d %d", &M, &N);
    Node* result = new Node();
    for (int i = 0; i < N; i++) {
        std::vector <short int> buffer;
        result->table.push_back(buffer);
        for (int j = 0; j < M; j++) {
            int temp;
            scanf("%d", &temp);
            result->table[i].push_back(temp);
        }
    }
    result->fitness = result->getNumberOfColors();
    return result;
}

int main() {
    freopen("flood-it.in", "r", stdin);
    freopen("flood-it.out", "w", stdout);
    Node* initial_state = input();
    StrategyPattern* factory = new StrategyPattern(initial_state);
    Algorithm* dummy = new Dummy();
    Algorithm* branch_and_bound = new BranchAndBound();
    factory->setAlgorithm(dummy);
    factory->optimize(100);
    //factory->setAlgorithm(branch_and_bound);
    //factory->optimize(100);
    factory->getBest()->print();
    return 0;
}
